'use strict';

/**
 * DOM manipulation functions
 */

function setStage(stage) {
	const name = document.getElementById("span-name");
	name.innerText = alloc.name;

	const elements = document.getElementsByClassName("stage");
	for ( let i=0; i<elements.length; i++ ) {
		const elm = elements[i];
		if (elm.id != stage)
			elm.classList.add('d-none');
		else
			elm.classList.remove('d-none');
	}
}

function removeAllChildren( elem ) {
	while( elem.firstChild ) {
		elem.removeChild( elem.firstChild );
	}
}

function logout() {
    window.localStorage.clear();
    window.location.reload();
}

function showAllocation(team) {
	const name = document.getElementById("span-team");
	name.innerText = team.id;

	const elmMembers = document.getElementById("span-members");
	removeAllChildren( elmMembers );
	for ( let member in team.members ) {
		let item = document.createElement("li");
		item.innerText = team.names[member];
		elmMembers.appendChild( item );
	}

	setStage('stage-allocated');
	const allocated = document.getElementById("stage-allocated");
	allocated.classList.add("reveal");
}

function overrideName(attr) {
	let newName = prompt("What's the new name?", attr.dataset.name );
	let studentID = attr.dataset.uuid;
	alloc.forceRename(studentID, newName);
}

function renameStudent(studentID) {
	let newName = prompt("What's the new name?", cache.students[studentID].name );
    if ( newName ) {
	    alloc.forceRename(studentID, newName);
    }
}

function notifyStudent(studentID) {
    let message = prompt("What's the message?", "");
    if ( message ) {
	    alloc.notifyStudent(studentID, message);
    }
}

function notifyGroup(groupID) {
    let message = prompt("What's the message?", "");
    if ( message ) {
	    alloc.notifyGroup(groupID, message);
    }
}

function kickStudent(studentID) {
	let message = confirm("Are you sure?");
    if ( message ) {
	    alloc.kickStudent(studentID);
    }
}

function changeName(){
	const newName = document.getElementById("input-name").value;
	const password = document.getElementById("input-password").value;

	alloc.setup( newName, password );
	return false;
}

function changeNameRemote(newName, role){
	const part = document.getElementById("stage-submitted");
	part.classList.remove("d-none");

	const name = document.getElementById("span-name");
	name.innerText = newName;

	if ( role == 1 ) {
		setStage( 'stage-wait' );
	} else if ( role == 2 ) {
		setStage( 'stage-staff' );
	}
	return false;
}

function showPassword() {
    let panel = document.querySelector(".staff-only");
    panel.classList.remove("d-none");
}

let studentCache = {};
let studentIDs = [];
function refreshStudents( names ){
	studentCache = {};
	studentIDs = [];

	for ( let student in names ) {
		const attr = names[ student ];
		studentCache[ attr.uuid ] = attr;
		studentIDs.push( attr.uuid );
	}
	rebuildStudents();
}

function rebuildStudents() {
	const container = document.getElementById("staff-students");
	removeAllChildren(container);

	const template = document.getElementById("tpl-student");

	cache.studentIds.forEach( function(studentID) {
		const student = cache.students[ studentID ];

		let attrElm = template.content.cloneNode(true);
		const attrTitle = attrElm.querySelector(".card-title");
		attrTitle.innerText = student.name;

		attrTitle.dataset.uuid = student.id;
		attrTitle.dataset.name = student.name || "Unknown student";
		attrTitle.addEventListener("click", function() {
			overrideName(attrTitle);
		});

        const notifyLink = attrElm.querySelector(".notify");
        notifyLink.addEventListener("click", function() {
			notifyStudent(student.id);
        });

        const renameLink = attrElm.querySelector(".rename");
        renameLink.addEventListener("click", function() {
			renameStudent(student.id);
        });


        const kickLink = attrElm.querySelector(".kick");
        kickLink.addEventListener("click", function() {
			kickStudent(student.id);
        });

		const card = attrElm.querySelector(".card");
		if ( !student.connected ) {
			card.classList.add('bg-warning');
			card.classList.add('text-white');
		} else {
			card.classList.remove('bg-warning');
			card.classList.remove('text-white');
		}

		if ( student.role == 128 ) {
			card.classList.add('bg-primary');
			card.classList.add('text-white');
		}

		container.appendChild( attrElm );
	});
}

function onStartAllocate() {
	let params = {
		"idealSize": document.getElementById("input-ideal").value,
		"minSize": document.getElementById("input-minsize").value,
		"method": document.getElementById("input-method").value
	};
	alloc.forceAllocate(params);
}

function resumeStaff() {
	// allow rebuilding of data from localStorage cache
	if ( window.localStorage.groups ){
		rebuildGroups(JSON.parse( window.localStorage.groups ));
	}
}

function showGroups(payload) {
	rebuildGroups(payload);

	// Persist group data
	let groupCache = [];
	for ( let groupID in payload ) {
		const group = payload[groupID];
		groupCache.push( group );
	}
	window.localStorage.groups = JSON.stringify(groupCache);
}

function rebuildGroups(groups) {
	const groupings = document.getElementById("staff-groupings");
	removeAllChildren( groupings );

	const template = document.getElementById("tpl-group");

	for ( let groupID in groups ) {
		const group = groups[groupID];
		cache['groups'].push( group );

		let groupElm = template.content.cloneNode(true);
		groupElm.querySelector(".card-title").innerText = "Group " + group.id;

		// notify link
		const notifyLink = groupElm.querySelector(".notify");
		notifyLink.addEventListener("click", function() {
				notifyGroup(group.id);
		});

		// students
		let list = groupElm.querySelector("ul");
		for ( let member in group.members ) {
			let memberElm = document.createElement("li");
			memberElm.classList.add( "list-group-item" );
			memberElm.innerText = group.names[member];
			list.appendChild(memberElm);
		}
		groupings.appendChild( groupElm );
	}
}

/**
 * Websocket functions
 */

let cache = {
	'students': {},
	'studentIds': [],
    'groups': []
};

let alloc = null;
class AllocatorToy {
	constructor(ws) {
		this.ws = ws;
		this.uuid = null;
		this.name = null;

		ws.onopen = this.onConnect;
		ws.onmessage = this.onMessage;
		ws.onclose = this.onError;
	}
	onConnect() {
	}
	onError(evt) {
		if ( evt.reason == "invalid ticket") {
			alert("invalid session, refresh page!");
			window.localStorage.clear();
		} else if ( evt.reason == "authentication failed") {
			alert("staff password was incorrect!");
		} else {
            if ( evt.reason ) {
			    alert( evt.reason );
            }
		}
		setStage('stage-error');
		console.log(evt);
	}
	onMessage(evt) {
		let msg = JSON.parse(evt.data);
		console.log(msg);
		switch ( msg.type ) {
			case 'welcome':
				if ( window.localStorage.token ) {
					alloc.resume( window.localStorage.token );
				}
				setStage('stage-name');
				break;
			case 'user':
				changeNameRemote( msg.data.name, msg.data.role );
				alloc.name = msg.data.name;
				alloc.role = msg.data.role;
				if ( alloc.role == 1 ) {
					setStage('stage-wait');
                } else if ( alloc.role == 128 ) {
					setStage('stage-staff');
				}

				if ( msg.data.ticket ) {
					window.localStorage.token = msg.data.ticket;
				}
				break;
			case 'kick':
				window.localStorage.clear();
                alloc.ws.close();
				setStage('stage-error');
                alert("kicked by server");
				break;
            case 'group':
			case 'allocate':
                // probably obsolete - replaced with groups
				showAllocation( msg.data );
				break;
			case 'token':
				if ( !window.localStorage.token ) {
					window.localStorage.token = msg.data;
					alloc.uuid = msg.data;
					setStage('stage-name');
				} else {
					alloc.resume( window.localStorage.token );
					alloc.uuid = msg.data;
					setStage('stage-name');
				}
				break;
			case 'status':
				const uid = msg.data.uid;
				if ( !cache.students[uid] ) {
					cache.studentIds.push( uid );
					cache.students[uid] = {
						'id': uid,
						'sid': msg.data.sid,
					};
				}
				if ( msg.data.connected ) {
					cache.students[uid].name = msg.data.name;
					cache.students[uid].role = msg.data.role;
				}
				cache.students[uid].connected = msg.data.connected;
				rebuildStudents();
				break;
			case 'users':
				msg.data.forEach( function(student) {
					console.log( student );
					const uid = student.uid;
					if ( !cache.students[uid] ) {
						cache.studentIds.push( uid );
						cache.students[uid] = {
							'id': uid,
							'sid': student.sid,
						};
					}
					if ( student.connected ) {
						cache.students[uid].name = student.name;
						cache.students[uid].role = student.role;
					}
					cache.students[uid].connected = student.connected;
				});
				rebuildStudents();
				break;
            case 'userRename':
                alloc.name = msg.data.name;
                const name = document.getElementById("span-name");
                name.innerText = alloc.name;

                // flag the user
                let header = document.getElementById("header");
                if ( header ) {
                    header.classList.remove("bg-dark");
                    header.classList.add("bg-danger");
                }
                break;
            case 'userRenameAdm': {
                    const uid = msg.data.uid;
                    if ( cache.students[uid] ) {
                        cache.students[uid].name = msg.data.name;
                    }
                    rebuildStudents();
                }
                break;
			case 'groups':
				showGroups( msg.data );
				break;
            case 'notify':
                alert( msg.data.message );
                break;
			case 'changeName':
				changeNameRemote( msg.data.name, msg.data.role );
				alloc.name = msg.data.name;
				alloc.role = msg.data.role;
				if ( alloc.role == 0 ) {
					setStage('stage-name');
				} else if ( alloc.role == 1 ) {
					setStage('stage-wait');
				} else if ( alloc.role == 2 ) {
					setStage('stage-staff');
				}
				break;
		}
	}
	send(type, payload) {
		const msg = {
			'type': type,
			'data': payload
		};
		this.ws.send( JSON.stringify( msg ) );
	}
	setup( name, password ) {
		if ( !alloc.ws ) {
			alloc.ws = new WebSocket('ws://localhost:8081');
		}

		let payload = { 'name': name };
		if ( password ) {
			payload['password'] = password;
		}
		this.send( 'setup', payload );
	}
	setName(name, password){
		let payload = { 'name': name };
		if ( password ) {
			payload['password'] = password;
		}
		this.send( 'changeName', payload );
	}
	resume(token){
		this.send( 'ticket', token );
	}
	forceAllocate(params) {
		this.send( 'allocate', params );
		console.log("allocation requested");
	}
	forceRename(uuid, name) {
		this.send( 'forceRename', { 'uuid': uuid, 'name': name } );
	}
    notifyStudent(uuid, message) {
		this.send( 'notify', { 'uuid': uuid, 'message': message } );
    }
    notifyGroup(uuid, message) {
		this.send( 'notifyGroup', { 'group': uuid, 'message': message } );
    }
    kickStudent(uuid) {
		this.send( 'kick', { 'uuid': uuid } );
    }
};


function start() {
	// FIXME: should be a better (non hard-coded) way to deal with this...
	const ws = new WebSocket('ws://alloc-api.fgxy.uk');
	alloc = new AllocatorToy( ws );

}

